<?php
/**
 * Copyright © 2018 Propoza. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace Propoza\Propoza\Helper;

use Magento\Framework\App\Helper\Context;

/**
 * Propoza data helper
 */
class Data extends \Magento\Framework\App\Helper\AbstractHelper
{
    /**
     * App state
     *
     * @var \Magento\Framework\App\State
     */
    private $appState;
    /**
     * @var string
     */
    private $domain = 'propoza.com';

    /**
     * @param \Magento\Framework\App\State $appState
     * @param Context $context
     */
    public function __construct(\Magento\Framework\App\State $appState, Context $context)
    {
        parent::__construct($context);
        $this->appState = $appState;
    }

    /**
     * Get url to sign-up page for Propoza
     *
     * @return string
     */
    public function getSignUpPropozaUrl()
    {
        return sprintf('%s%s/accounts/create?client=magento2', $this->getProtocol(), $this->getPropozaDomain());
    }

    /**
     * Get protocol used for propoza
     *
     * @return string
     */
    public function getProtocol()
    {
        return 'https://';
    }

    /**
     * Get base domain for Propoza
     *
     * @return string
     */
    public function getPropozaDomain()
    {
        return $this->domain;
    }

    /**
     * @param null $subDomain
     * @param null $token
     * @return string
     */
    public function getDashboardPropozaLoginUrl($subDomain = null, $token = null)
    {
        if (isset($token)) {
            $token = sprintf('?_token=%s', $token);
        }
        return sprintf('%s/login%s', $this->getDashboardPropozaUrl($subDomain), $token);
    }

    /**
     * Get url to dashboard constructed with either given parameter or else the stored sub-domain
     *
     * @param  null $subDomain
     * @return string
     */
    public function getDashboardPropozaUrl($subDomain = null)
    {
        if ($subDomain === null
        ) {
            $subDomain = $this->scopeConfig->getValue('propoza/general/sub_domain');
        }

        return sprintf('%s%s.%s', $this->getProtocol(), $subDomain, $this->getPropozaDomain());
    }

    /**
     * @param null $subDomain
     * @return string
     */
    public function getDashboardPropozaTokenUrl($subDomain = null)
    {
        return sprintf('%s/api/MerchantsApi/token.json', $this->getDashboardPropozaUrl($subDomain));
    }

    /**
     * Get url to test a connection to Propoza
     *
     * @param  null $subDomain
     * @return string
     */
    public function getConnectionTestUrl($subDomain = null)
    {
        return sprintf('%s/api/Magento2Quotes/testConnection.json', $this->getDashboardPropozaUrl($subDomain));
    }

    /**
     * Get url to Propoza from where to get the Quote request form
     *
     * @return string
     */
    public function getQuoteRequestFormUrl()
    {
        return sprintf('%s/api/Magento2Quotes/quoteRequestForm', $this->getDashboardPropozaUrl(null));
    }

    /**
     * Get url to Propoza for the Quick Quote button
     *
     * @return string
     */
    public function getQuoteButtonUrl()
    {
        return sprintf('%s/api/Magento2Quotes/requestButton', $this->getDashboardPropozaUrl(null));
    }

    /**
     * Get url to Propoza for the Quick Quote button
     *
     * @return string
     */
    public function getQuickQuoteButtonUrl()
    {
        return sprintf('%s/api/Magento2Quotes/requestQuickQuoteButton', $this->getDashboardPropozaUrl(null));
    }

    /**
     * Get url to Propoza for the Quick Quote form
     *
     * @return string
     */
    public function getQuickQuoteUrl()
    {
        return sprintf('%s/api/Magento2Quotes/requestQuickQuote', $this->getDashboardPropozaUrl(null));
    }
    /**
     * Check if request is authorized
     *
     * @return bool
     */
    public function isRequestAuthorized()
    {
        return true;
    }

    /**
     * Check for validity api key.
     * Should be alphanumeric, 226 characters long and end with ==
     *
     * @param  $string
     * @return int
     */
    public function isValidApiKey($string)
    {
        return preg_match('/^[A-Za-z0-9+\/]{226}==$/', $string);
    }

    /**
     * @return bool
     */
    public function isEnabledAndConfigured()
    {
        return $this->isEnabled() && $this->isConfigured();
    }

    /**
     * Get sub_domain from store config
     *
     * @return bool
     */
    public function isEnabled()
    {
        return (bool)$this->scopeConfig->getValue('propoza/general/enabled');
    }

    /**
     * Check if configuration values are set to make a request to Propoza
     *
     * @return bool
     */
    public function isConfigured()
    {
        $apiKey = $this->getApiKey();
        $subDomain = $this->getSubDomain();

        return (isset($apiKey) && isset($subDomain) && trim($apiKey) !== '' && trim($subDomain) !== '');
    }

    /**
     * Get api_key from store config
     *
     * @return mixed
     */
    public function getApiKey()
    {
        return $this->scopeConfig->getValue('propoza/general/api_key');
    }

    /**
     * Get sub_domain from store config
     *
     * @return mixed
     */
    public function getSubDomain()
    {
        return $this->scopeConfig->getValue('propoza/general/sub_domain');
    }

    /**
     * @return bool
     */
    public function isDeveloperMode()
    {
        return $this->appState->getMode() == \Magento\Framework\App\State::MODE_DEVELOPER;
    }


    /**
     * Get url to set quote as ordered request to
     *
     * @return string
     */
    public function getQuoteOrderedUrl()
    {
        return sprintf('%s/api/Magento2Quotes/edit.json', $this->getDashboardPropozaUrl());
    }
}
