<?php
/**
 * Copyright © 2018 Propoza. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace Propoza\Propoza\Helper;

/**
 * Propoza Request helper
 */
class Request extends \Magento\Framework\App\Helper\AbstractHelper
{
    /**
     * @var \Propoza\Propoza\Helper\Data
     */
    private $propozaHelper;
    /**
     * @var \Magento\Checkout\Model\Session
     */
    private $checkoutSession;
    /**
     * @var \Magento\Quote\Model\Quote
     */
    private $currentQuote;
    /**
     * @var \Magento\Customer\Model\Customer
     */
    private $currentCustomer;
    /**
     * @var \Magento\Directory\Model\Country
     */
    private $countryModel;
    /**
     * @var \Magento\Tax\Model\Config
     */
    private $taxConfig;
    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    private $storeManager;
    /**
     * @var \Magento\Customer\Model\ResourceModel\CustomerRepository
     */
    private $customerRepository;

    /**
     * @param \Magento\Framework\App\Helper\Context $context
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param \Magento\Directory\Model\Country $country
     * @param \Magento\Customer\Model\ResourceModel\CustomerRepository $customerRepository
     * @param Data $propozaHelper
     * @param \Magento\Checkout\Model\Session $checkoutSession
     * @param \Magento\Tax\Model\Config $taxConfig
     */
    public function __construct(
        \Magento\Framework\App\Helper\Context $context,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Directory\Model\Country $country,
        \Magento\Customer\Model\ResourceModel\CustomerRepository $customerRepository,
        \Propoza\Propoza\Helper\Data $propozaHelper,
        \Magento\Checkout\Model\Session $checkoutSession,
        \Magento\Tax\Model\Config $taxConfig
    ) {
        parent::__construct($context);
        $this->propozaHelper = $propozaHelper;
        $this->checkoutSession = $checkoutSession;
        $this->countryModel = $country;
        $this->taxConfig = $taxConfig;
        $this->storeManager = $storeManager;
        $this->currentQuote = $this->checkoutSession->getQuote();
        $this->customerRepository = $customerRepository;
        if ($customerId = $this->currentQuote->getCustomerId()) {
            $this->currentCustomer = $customerRepository->getById($customerId);
        } else {
            $this->currentCustomer = $this->currentQuote->getCustomer();
        }
    }

    /**
     * Prepare the quote request for the request
     * @param array $item
     * @return array
     */
    public function prepareQuoteRequest($quote = null, $image = null)
    {
        if (!isset($quote)) {
            $quote = $this->checkoutSession->getQuote();
        }
        $toPropozaQuote = null;
        $toPropozaQuote['Quote'] = $this->prepareQuote($quote);
        if (isset($image)) {
            $toPropozaQuote['Quote']['image'] = $image;
        }
        $toPropozaQuote['Quote']['Product'] = $this->prepareProducts($quote);


        $toPropozaQuote['Quote']['shop_url'] = $this->storeManager->getStore()->getBaseUrl();

        return $toPropozaQuote;
    }

    /**
     * Prepare the quote for the request
     *
     * @param $quote
     *
     * @return array
     */
    private function prepareQuote($quote)
    {
        $toPropozaQuote = null;
        $toPropozaQuote ['shop_quote_id'] = $quote->getId();
        $toPropozaQuote['cart_currency'] = $quote->getQuoteCurrencyCode();
        $toPropozaQuote['include_default_store_tax'] = $this->taxConfig->priceIncludesTax();

        return $toPropozaQuote;
    }

    /**
     * Prepare the products for the request
     *
     * @param  $quote
     * @return array
     */
    private function prepareProducts($quote)
    {
        $products = null;
        $count = 0;
        foreach ($quote->getAllVisibleItems() as $item) {
            $products[$count] = $this->prepareProduct($item);
            $products[$count]['Child'] = $this->prepareChildren($item);
            $count++;
        }

        return $products;
    }

    /**
     * Prepare the product for the request
     *
     * @param  $item
     * @return array
     */
    private function prepareProduct($item, $child = false)
    {
        $product = null;
        $product['name'] = $item->getName();
        if ($child === false) {
            $product['original_price'] = $item->getCalculationPrice();
        }
        $product['sku'] = $item->getSku();
        $product['quantity'] = $item->getQty();
        $product['ProductAttribute'] = $this->prepareProductAttributes($item);
        $product['external_url'] = $item->getProduct()->getProductUrl();
        return $product;
    }

    /**
     * Prepare the attributes for the request
     *
     * @param  $item
     * @return array
     */
    private function prepareProductAttributes($item)
    {
        $productAttributes = null;
        $orderOptions = ($item->getProduct()->getTypeInstance(true)->getOrderOptions($item->getProduct()));
        $counter = 0;
        if (isset($orderOptions['options'])) {
            foreach ($orderOptions['options'] as $option) {
                $productAttributes[$counter]['name'] = $option['label'];
                $productAttributes[$counter]['value'] = $option['print_value'];
                $counter++;
            }
        }
        if (isset($orderOptions['attributes_info'])) {
            foreach ($orderOptions['attributes_info'] as $attribute) {
                $productAttributes[$counter]['name'] = $attribute['label'];
                $productAttributes[$counter]['value'] = $attribute['value'];
                $counter++;
            }
        }

        return $productAttributes;
    }

    /**
     * Prepare the child products for the request
     *
     * @param  $parent
     * @return array
     */
    private function prepareChildren($parent)
    {
        $children = null;
        if (count($parent->getChildren()) > 0) {
            foreach ($parent->getChildren() as $child) {
                $children[] = $this->prepareProduct($child, true);
            }
        }

        return $children;
    }

    /**
     * Prepare the current logged in user for the request
     *
     * @return mixed
     */
    public function getPreparedLoggedInUser()
    {
        $loggedInUser['Requester'] = $this->prepareRequester();

        return $loggedInUser;
    }

    /**
     * Prepare the requester for the request
     *
     * @return array
     */
    public function prepareRequester()
    {
        $requester = null;
        $requester['firstname'] = $this->currentCustomer->getFirstname();
        $requester['lastname'] = trim(
            sprintf('%s %s', $this->currentCustomer->getMiddlename(), $this->currentCustomer->getLastname())
        );
        $requester['email'] = $this->currentCustomer->getEmail();
        $requester['Address'] = $this->prepareAddress();

        return $requester;
    }

    /**
     * Prepare the address for the request
     *
     * @return array
     */
    private function prepareAddress()
    {

        $defaultAddress = $this->currentCustomer->getDefaultBillingAddress();
        if (!$defaultAddress) {
            $defaultAddress = $this->currentCustomer->getDefaultShippingAddress();
        }
        $address = null;
        if ($defaultAddress) {
            $address['company'] = $defaultAddress->getCompany();
            $address['telephone'] = $defaultAddress->getTelephone();
            $address['zipcode'] = $defaultAddress->getPostcode();
            $address['city'] = $defaultAddress->getCity();
            $address['country'] = $this->countryModel->loadByCode($defaultAddress->getCountry())->getName();
            $address['street_address'] = $defaultAddress->getStreetFull();
            $address['state'] = $defaultAddress->getRegion();
        }

        return $address;
    }
}
