<?php
/**
 * Copyright © 2018 Propoza. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace Propoza\Propoza\Helper;

/**
 * Propoza Rule helper
 */
class Rule extends \Magento\Framework\App\Helper\AbstractHelper
{
    /**
     * @var \Magento\SalesRule\Model\RuleFactory
     */
    private $ruleFactory;
    /**
     * @var \Magento\SalesRule\Model\Rule\Condition\AddressFactory
     */
    private $addressConditionFactory;
    /**
     * @var \Magento\Customer\Model\ResourceModel\Group\Collection
     */
    private $customerGroupCollection;
    /**
     * @var \Magento\Framework\DataObjectFactory
     */
    private $objectFactory;
    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    private $storeManager;
    /**
     * @var \Magento\SalesRule\Model\CouponFactory
     */
    private $couponFactory;
    /**
     * @var \Magento\SalesRule\Model\ResourceModel\Rule
     */
    private $ruleResourceModel;
    /**
     * @var \Magento\SalesRule\Model\ResourceModel\Rule\Collection
     */
    private $ruleCollection;

    /**
     * Rule constructor.
     * @param \Magento\SalesRule\Model\RuleFactory $ruleFactory
     * @param \Magento\SalesRule\Model\ResourceModel\Rule $ruleResourceModel
     * @param \Magento\SalesRule\Model\ResourceModel\Rule\Collection $ruleCollection
     * @param \Magento\SalesRule\Model\Rule\Condition\AddressFactory $addressConditionFactory
     * @param \Magento\Customer\Model\ResourceModel\Group\Collection $customerGroupCollection
     * @param \Magento\Framework\DataObjectFactory $objectFactory
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param \Magento\SalesRule\Model\CouponFactory $couponFactory
     * @param \Magento\Framework\App\Helper\Context $context
     */
    public function __construct(
        \Magento\SalesRule\Model\RuleFactory $ruleFactory,
        \Magento\SalesRule\Model\ResourceModel\Rule $ruleResourceModel,
        \Magento\SalesRule\Model\ResourceModel\Rule\Collection $ruleCollection,
        \Magento\SalesRule\Model\Rule\Condition\AddressFactory $addressConditionFactory,
        \Magento\Customer\Model\ResourceModel\Group\Collection $customerGroupCollection,
        \Magento\Framework\DataObjectFactory $objectFactory,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\SalesRule\Model\CouponFactory $couponFactory,
        \Magento\Framework\App\Helper\Context $context
    ) {
        parent::__construct($context);
        $this->ruleFactory = $ruleFactory;
        $this->addressConditionFactory = $addressConditionFactory;
        $this->customerGroupCollection = $customerGroupCollection;
        $this->objectFactory = $objectFactory;
        $this->storeManager = $storeManager;
        $this->couponFactory = $couponFactory;
        $this->ruleResourceModel = $ruleResourceModel;
        $this->ruleCollection = $ruleCollection;
    }

    /**
     * Create a sales rule for Propoza proposal
     *
     * @param  $quote
     * @param  $propozaQuoteId
     * @param  $discount
     * @param  $baseSubTotal
     * @return \Magento\SalesRule\Model\Rule
     * @throws \Magento\Framework\Exception\ValidatorException
     * @throws \Exception
     */
    public function createProposalRule($quote, $propozaQuoteId, $discount, $baseSubTotal)
    {
        $rule = $this->ruleFactory->create();
        $couponCode = $rule->getCouponCodeGenerator()->generateCode();
        $actionConditionSubTotal = $this->addressConditionFactory->create()
            ->setAttribute('base_subtotal')
            ->setOperator('==')
            ->setValue($baseSubTotal)
            ->setType('Magento\SalesRule\Model\Rule\Condition\Address');
        $actionConditionQuoteId = $this->addressConditionFactory->create()
            ->setAttribute('quote_id')
            ->setOperator('==')
            ->setValue($quote->getId())
            ->setType('Magento\SalesRule\Model\Rule\Condition\Address');

        $customerGroupIds = $this->customerGroupCollection->addFieldToSelect('customer_group_id')->load()->getAllIds();
        $rule->setName(__(sprintf('Propoza proposal for quote #%s', $propozaQuoteId)));
        $rule->setCouponType(\Magento\SalesRule\Model\Rule::COUPON_TYPE_SPECIFIC);
        $rule->setCouponCode($couponCode);
        $rule->setPropozaQuoteId($propozaQuoteId);
        $rule->setUsesPerCoupon(true);
        $rule->setUsesPerCustomer(true);
        $rule->setCustomerGroupIds($customerGroupIds);
        $rule->setIsActive(true);
        $rule->getConditions()->addCondition($actionConditionSubTotal);
        $rule->getConditions()->addCondition($actionConditionQuoteId);
        $rule->setDescription(null);
        $rule->setFromDate(null);
        $rule->setToDate(null);
        $rule->setProductIds(null);
        $rule->setSortOrder(null);
        $rule->setSimpleAction(\Magento\SalesRule\Model\Rule::CART_FIXED_ACTION);
        $rule->setDiscountAmount($discount);
        $rule->setDiscountQty(false);
        $rule->setDiscountStep(false);
        $rule->setApplyToShipping(false);
        $rule->setStopRulesProcessing(true);
        $rule->setTimesUsed(false);
        $rule->setIsRss(false);
        $rule->setSimpleFreeShipping(false);
        $rule->setWebsiteIds($this->storeManager->getStore(true)->getWebsiteId());
        $rule->setStoreLabels(['Proposal']);
        $validateResult = $rule->validateData($this->objectFactory->create($rule->getData()));
        if ($validateResult !== true) {
            throw new \Magento\Framework\Exception\ValidatorException(__('Coupon rule not validated'));
        }

        $rule->save();

        return $rule;
    }

    /**
     * Update the Propoza proposal rule
     * @param \Magento\SalesRule\Model\Rule $rule
     * @param $quote
     * @param $proposalTotal
     * @param $baseSubTotal
     * @return \Magento\SalesRule\Model\Rule
     * @throws \Magento\Framework\Exception\ValidatorException
     * @throws \Exception
     */
    public function updateProposalRule($rule, $quote, $proposalTotal, $baseSubTotal)
    {
        $actionConditionSubTotal = $this->addressConditionFactory->create()
            ->setAttribute('base_subtotal')
            ->setOperator('==')
            ->setValue($baseSubTotal)
            ->setType('Magento\SalesRule\Model\Rule\Condition\Address');
        $actionConditionQuoteId = $this->addressConditionFactory->create()
            ->setAttribute('quote_id')
            ->setOperator('==')
            ->setValue($quote->getId())
            ->setType('Magento\SalesRule\Model\Rule\Condition\Address');

        $rule->setCouponCode($rule->getPrimaryCoupon()->getCode());
        $rule->getConditions()->setConditions(null);
        $rule->getConditions()->addCondition($actionConditionSubTotal);
        $rule->getConditions()->addCondition($actionConditionQuoteId);
        $rule->setDiscountAmount($proposalTotal);
        $validateResult = $rule->validateData($this->objectFactory->create($rule->getData()));
        if ($validateResult !== true) {
            throw new \Magento\Framework\Exception\ValidatorException(__('Coupon rule not validated'));
        }
        $this->ruleResourceModel->save($rule);

        return $rule;
    }

    /**
     * Check if salesrule has a propoza quote id
     *
     * @param  $couponCode
     * @return bool
     */
    public function isPropozaProposalRule($couponCode)
    {
        return $this->getSalesRuleByCouponCode($couponCode)->getPropozaQuoteId() != null;
    }

    /**
     * Get sales rule by coupon code
     *
     * @param  $couponCode
     * @return \Magento\SalesRule\Model\Rule
     */
    public function getSalesRuleByCouponCode($couponCode)
    {
        $ruleId = $this->couponFactory->create()->loadByCode($couponCode)->getRuleId();
        $rule = $this->ruleFactory->create();
        $this->ruleResourceModel->load($rule, $ruleId);

        return $rule;
    }

    /**
     * @param $quoteId
     * @return \Magento\SalesRule\Model\ResourceModel\Rule\Collection
     */
    public function getCouponseByPropozaQuoteId($quoteId)
    {
        return $this->ruleCollection->addFieldToFilter('propoza_quote_id', $quoteId);
    }
}
